using System;
using System.IO;
using System.Collections;
using Mono.Math.ForIronPython;

namespace BigIntFactor
{

class bigFactor
   {
   // ------------------------------------------------------------------------
   //   Binary power function 
   // ------------------------------------------------------------------------
   public static BigInteger bubPow (BigInteger b, uint e, BigInteger m)
   {
      if( e == 0 )   return(new BigInteger(1) );
      if( e == 1 )   return(b % m);

      BigInteger bpower = b;
      BigInteger prod   = 1;
      while(e != 0) {
        if( (e%2) == 1 )  {
           prod = (prod * bpower) % m;
        }
        bpower = (bpower * bpower) % m;
        e = e >> 1;
      } 
      return(prod);
   } // bubPow(big,int,big)

   // ------------------------------------------------------------------------
   public static BigInteger cvtString( string s) {
      //Console.Out.WriteLine("cvtString input = "+ s);
      BigInteger t = 0;
      //Console.Out.WriteLine("cvtString length = "+ s.Length);
      if( s.Length < 10 ) {
         t = uint.Parse(s);
         return(t);
      }
      int firstCut = s.Length % 9;
      int numberCuts = s.Length / 9;
      //Console.Out.WriteLine("cvtString firstCut = "+ firstCut);
      //Console.Out.WriteLine("cvtString numberCuts = "+ numberCuts);
      if( firstCut > 0) {
        string cutString = s.Substring(0,firstCut);
        t = uint.Parse(cutString);
        //Console.Out.WriteLine("first cut  "+ t.showMe());
      }
      for( int pos=firstCut; pos < s.Length; pos += 9 ) {
         string cutString = s.Substring(pos, 9);
         //Console.Out.WriteLine("next cut   "+ cutString);
         t = 1000000000 * t + uint.Parse(cutString) ;
         //Console.Out.WriteLine("running val"+ t.showMe());
      }
      return(t);
    }

} // class bigFactor

// ---------------------------------------------------------------------------
class bigTest {
   [STAThread]
   public static void Main(string[] args)  {

// 8069490149.modPower(3,11522063093) =

      BigInteger b = bigFactor.cvtString("8069490149");
      BigInteger e = 3;
      BigInteger m = bigFactor.cvtString("11522063093");
      BigInteger x = b.modPow(e,m);
      BigInteger y = bigFactor.bubPow(b,3,m);

      Console.Out.WriteLine(
         String.Format("modPow:  {0}^{1} % {2} = {3} ", b, e, m, x)
      );
      Console.Out.WriteLine(
         String.Format("bubPow:  {0}^{1} % {2} = {3} ", b, e, m, y)
      );

      Console.Out.WriteLine("Check");

      BigInteger b3 = b*b*b;
      Console.Out.WriteLine(
         String.Format("base^3:       {0}", b3)
      );

      BigInteger bDiv = b3 / m;
      Console.Out.WriteLine(
         String.Format("base^3 / modulus:       {0}", bDiv)
      );

      BigInteger bProd = bDiv * m;
      Console.Out.WriteLine(
         String.Format("* modulus:    {0}", bProd)
      );

      BigInteger bDiff = b3 - bProd;
      Console.Out.WriteLine(
         String.Format("Difference:                       {0}", bDiff)
      );

   } // bigTest::Main()
} // class bigTest

} // namespace BigIntFactor

